@@ -1,6 +1,3 @@
-/* $XdotOrg: xc/lib/font/fontfile/fontdir.c,v 1.4 2005/07/03 07:01:00 daniels Exp $ */
-/* $Xorg: fontdir.c,v 1.4 2001/02/09 02:04:03 xorgcvs Exp $ */
-
 /*
 
 Copyright 1991, 1998  The Open Group
@@ -26,7 +23,6 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/font/fontfile/fontdir.c,v 3.22 2003/07/07 16:40:11 eich Exp $ */
 
 /*
  * Author:  Keith Packard, MIT X Consortium
@@ -47,11 +43,11 @@ in this Software without prior written authorization from The Open Group.
 Bool
 FontFileInitTable (FontTablePtr table, int size)
 {
-    if (size < 0 || (size > INT32_MAX/sizeof(FontEntryRec))) 
+    if (size < 0 || (size > INT32_MAX/sizeof(FontEntryRec)))
 	return FALSE;
     if (size)
     {
-	table->entries = (FontEntryPtr) xalloc(sizeof(FontEntryRec) * size);
+	table->entries = malloc(sizeof(FontEntryRec) * size);
 	if (!table->entries)
 	    return FALSE;
     }
@@ -70,32 +66,28 @@ FontFileFreeEntry (FontEntryPtr entry)
     int i;
 
     if (entry->name.name)
-	xfree(entry->name.name);
+	free(entry->name.name);
     entry->name.name = NULL;
 
     switch (entry->type)
     {
     case FONT_ENTRY_SCALABLE:
-	xfree (entry->u.scalable.fileName);
+	free (entry->u.scalable.fileName);
 	extra = entry->u.scalable.extra;
 	for (i = 0; i < extra->numScaled; i++)
 	    if (extra->scaled[i].vals.ranges)
-		xfree (extra->scaled[i].vals.ranges);
-	xfree (extra->scaled);
-	xfree (extra);
+		free (extra->scaled[i].vals.ranges);
+	free (extra->scaled);
+	free (extra);
 	break;
     case FONT_ENTRY_BITMAP:
-	xfree (entry->u.bitmap.fileName);
+	free (entry->u.bitmap.fileName);
 	entry->u.bitmap.fileName = NULL;
 	break;
     case FONT_ENTRY_ALIAS:
-	xfree (entry->u.alias.resolved);
+	free (entry->u.alias.resolved);
 	entry->u.alias.resolved = NULL;
 	break;
-#ifdef NOTYET
-    case FONT_ENTRY_BC:
-	break;
-#endif
     }
 }
 
@@ -106,19 +98,19 @@ FontFileFreeTable (FontTablePtr table)
 
     for (i = 0; i < table->used; i++)
 	FontFileFreeEntry (&table->entries[i]);
-    xfree (table->entries);
+    free (table->entries);
 }
 
 FontDirectoryPtr
-FontFileMakeDir(char *dirName, int size)
+FontFileMakeDir(const char *dirName, int size)
 {
     FontDirectoryPtr	dir;
     int			dirlen;
     int			needslash = 0;
-    char		*attrib;
+    const char		*attrib;
     int			attriblen;
 
-#if !defined(__UNIXOS2__) && !defined(WIN32)
+#if !defined(WIN32)
     attrib = strchr(dirName, ':');
 #else
     /* OS/2 uses the colon in the drive letter descriptor, skip this */
@@ -136,19 +128,19 @@ FontFileMakeDir(char *dirName, int size)
     if (dirlen)     /* leave out slash for builtins */
 #endif
 	needslash = 1;
-    dir = (FontDirectoryPtr) xalloc(sizeof *dir + dirlen + needslash + 1 +
-				    (attriblen ? attriblen + 1 : 0));
+    dir = malloc(sizeof *dir + dirlen + needslash + 1 +
+		 (attriblen ? attriblen + 1 : 0));
     if (!dir)
 	return (FontDirectoryPtr)0;
     if (!FontFileInitTable (&dir->scalable, 0))
     {
-	xfree (dir);
+	free (dir);
 	return (FontDirectoryPtr)0;
     }
     if (!FontFileInitTable (&dir->nonScalable, size))
     {
 	FontFileFreeTable (&dir->scalable);
-	xfree (dir);
+	free (dir);
 	return (FontDirectoryPtr)0;
     }
     dir->directory = (char *) (dir + 1);
@@ -172,7 +164,7 @@ FontFileFreeDir (FontDirectoryPtr dir)
 {
     FontFileFreeTable (&dir->scalable);
     FontFileFreeTable (&dir->nonScalable);
-    xfree(dir);
+    free(dir);
 }
 
 FontEntryPtr
@@ -185,9 +177,13 @@ FontFileAddEntry(FontTablePtr table, FontEntryPtr prototype)
     if (table->sorted)
 	return (FontEntryPtr) 0;    /* "cannot" happen */
     if (table->used == table->size) {
+	if (table->size >= ((INT32_MAX / sizeof(FontEntryRec)) - 100))
+	    /* If we've read so many entries we're going to ask for 2gb
+	       or more of memory, something is so wrong with this font
+	       directory that we should just give up before we overflow. */
+	    return NULL;
 	newsize = table->size + 100;
-	entry = (FontEntryPtr) xrealloc(table->entries,
-					   newsize * sizeof(FontEntryRec));
+	entry = realloc(table->entries, newsize * sizeof(FontEntryRec));
 	if (!entry)
 	    return (FontEntryPtr)0;
 	table->size = newsize;
@@ -195,7 +191,7 @@ FontFileAddEntry(FontTablePtr table, FontEntryPtr prototype)
     }
     entry = &table->entries[table->used];
     *entry = *prototype;
-    entry->name.name = (char *) xalloc(prototype->name.length + 1);
+    entry->name.name = malloc(prototype->name.length + 1);
     if (!entry->name.name)
 	return (FontEntryPtr)0;
     memcpy (entry->name.name, prototype->name.name, prototype->name.length);
@@ -288,7 +284,7 @@ FontFileSortDir(FontDirectoryPtr dir)
 #define isDigit(c)  (XK_0 <= (c) && (c) <= XK_9)
 
 static int
-SetupWildMatch(FontTablePtr table, FontNamePtr pat, 
+SetupWildMatch(FontTablePtr table, FontNamePtr pat,
 	       int *leftp, int *rightp, int *privatep)
 {
     int         nDashes;
@@ -434,20 +430,16 @@ FontFileCountDashes (char *name, int namelen)
     return ndashes;
 }
 
+/* exported in public API in <X11/fonts/fntfil.h> */
 char *
 FontFileSaveString (char *s)
 {
-    char    *n;
-
-    n = (char *) xalloc (strlen (s) + 1);
-    if (!n)
-	return 0;
-    strcpy (n, s);
-    return n;
+    return strdup(s);
 }
+#define FontFileSaveString(s) strdup(s)
 
 FontEntryPtr
-FontFileFindNameInScalableDir(FontTablePtr table, FontNamePtr pat, 
+FontFileFindNameInScalableDir(FontTablePtr table, FontNamePtr pat,
 			      FontScalablePtr vals)
 {
     int         i,
@@ -457,6 +449,8 @@ FontFileFindNameInScalableDir(FontTablePtr table, FontNamePtr pat,
                 private;
     FontNamePtr	name;
 
+    if (!table->entries)
+	return NULL;
     if ((i = SetupWildMatch(table, pat, &start, &stop, &private)) >= 0)
 	return &table->entries[i];
     for (i = start; i < stop; i++) {
@@ -498,7 +492,7 @@ FontFileFindNameInDir(FontTablePtr table, FontNamePtr pat)
 }
 
 int
-FontFileFindNamesInScalableDir(FontTablePtr table, FontNamePtr pat, int max, 
+FontFileFindNamesInScalableDir(FontTablePtr table, FontNamePtr pat, int max,
 			       FontNamesPtr names, FontScalablePtr vals,
 			       int alias_behavior, int *newmax)
 {
@@ -585,7 +579,7 @@ FontFileFindNamesInScalableDir(FontTablePtr table, FontNamePtr pat, int max,
 }
 
 int
-FontFileFindNamesInDir(FontTablePtr table, FontNamePtr pat, 
+FontFileFindNamesInDir(FontTablePtr table, FontNamePtr pat,
 		       int max, FontNamesPtr names)
 {
     return FontFileFindNamesInScalableDir(table, pat, max, names,
@@ -693,7 +687,7 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 	    return FALSE;
 	if (!(bitmap = FontFileAddEntry (&dir->nonScalable, &entry)))
 	{
-	    xfree (entry.u.bitmap.fileName);
+	    free (entry.u.bitmap.fileName);
 	    return FALSE;
 	}
     }
@@ -721,7 +715,7 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 		{
 		    existing->u.scalable.extra->defaults = vals;
 
-		    xfree (existing->u.scalable.fileName);
+		    free (existing->u.scalable.fileName);
 		    if (!(existing->u.scalable.fileName = FontFileSaveString (fileName)))
 			return FALSE;
 		}
@@ -736,10 +730,10 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 	}
 	if (!(entry.u.scalable.fileName = FontFileSaveString (fileName)))
 	    return FALSE;
-	extra = (FontScalableExtraPtr) xalloc (sizeof (FontScalableExtraRec));
+	extra = malloc (sizeof (FontScalableExtraRec));
 	if (!extra)
 	{
-	    xfree (entry.u.scalable.fileName);
+	    free (entry.u.scalable.fileName);
 	    return FALSE;
 	}
 	bzero((char *)&extra->defaults, sizeof(extra->defaults));
@@ -750,10 +744,11 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 	{
 	    FontResolutionPtr resolution;
 	    int num;
+	    int default_point_size = GetDefaultPointSize();
 
 	    extra->defaults.point_matrix[0] =
 		extra->defaults.point_matrix[3] =
-		    (double)GetDefaultPointSize() / 10.0;
+	            (double)default_point_size / 10.0;
 	    extra->defaults.point_matrix[1] =
 		extra->defaults.point_matrix[2] = 0.0;
 	    extra->defaults.values_supplied =
@@ -773,7 +768,7 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 		    extra->defaults.y = 75;
 	        }
 	     }
-	     else 
+	     else
 	     {
 		extra->defaults.x = vals.x;
 		extra->defaults.y = vals.y;
@@ -789,8 +784,8 @@ FontFileAddFontFile (FontDirectoryPtr dir, char *fontName, char *fileName)
 	entry.u.scalable.extra = extra;
 	if (!(scalable = FontFileAddEntry (&dir->scalable, &entry)))
 	{
-	    xfree (extra);
-	    xfree (entry.u.scalable.fileName);
+	    free (extra);
+	    free (entry.u.scalable.fileName);
 	    return FALSE;
 	}
 	if (vals.values_supplied & SIZE_SPECIFY_MASK)
@@ -824,7 +819,7 @@ FontFileAddFontAlias (FontDirectoryPtr dir, char *aliasName, char *fontName)
 	return FALSE;
     if (!FontFileAddEntry (&dir->nonScalable, &entry))
     {
-	xfree (entry.u.alias.resolved);
+	free (entry.u.alias.resolved);
 	return FALSE;
     }
     return TRUE;
